<?php
/**
 * @file
 * Subs pages.
 */

/**
 * Page callback.
 * Displays page to select subscription type when adding new subscription.
 *
 * @see subs_menu()
 */
function subs_add_page() {
  $item = menu_get_item();
  $content = system_admin_menu_block($item);
  $count = count($content);

  // Bypass the subscription/add listing if only one subscription type is available.
  if ($count == 1) {
    $item = array_shift($content);
    drupal_goto($item['href']);
  }
  // Display warning message if no subscription types are available.
  elseif ($count === 0) {
    return t("You haven't created any !types yet!", array('!types' => l(t('Subscription Types'), 'admin/structure/subscriptions')));
  }

  return theme('node_add_list', array('content' => $content));
}

/**
 * Page callback.
 * Displays add new subscription page for the selected type.
 *
 * @see subs_menu()
 */
function subs_add($type) {
  $subscription_type = subs_get_types($type);
  $subscription = entity_create('subs', array('type' => $type));

  // Check if user can create a subscription of this type.
  if (!subs_access('create', $subscription)) {
    drupal_set_message(t("You can't create any more subscriptions."), 'error', FALSE);
    drupal_access_denied();
    exit;
  }

  drupal_set_title(t('Create @name Subscription', array('@name' => $subscription_type->getTranslation('label'))));
  $output = drupal_get_form('subs_form', $subscription);
  return $output;
}

/**
 * Form callback.
 * Generates the subscription form.
 *
 * @see subs_menu()
 */
function subs_form($form, &$form_state, $subscription) {
  $form_state['subscription'] = $subscription;
  $subscription_type = subs_get_types($subscription->type);
  $aid = entity_id('subs', $subscription);
  $administer_subs = user_access('administer subs');

  // Only show Start Date if user has permission.
  if ($administer_subs) {
    $form['subs_start'] = array(
      '#type' => 'fieldset',
      '#title' => t('Start date'),
      '#collapsible' => FALSE,
      '#weight' => 2,
    );

    $form['subs_start']['auto'] = array(
      '#title' => t('Start automatically when subscription becomes active'),
      '#description' => t('Caution: If the subscription is already active and you check this, the start date will be recalculated from today.'),
      '#type' => 'checkbox',
      '#default_value' => $subscription->date_start == 0,
    );

    if ($subscription->date_start != 0) {
      $date_start = date('Y-m-d', $subscription->date_start);
    }
    else {
      $date_start = date('Y-m-d');
    }

    $date_start = explode('-', $date_start);
    $date_start = array(
      'year' => (int) $date_start[0],
      'month' => (int) $date_start[1],
      'day' => (int) $date_start[2],
    );

    $form['subs_start']['date_start'] = array(
      '#type' => 'date',
      '#title' => t('Start on a specific date'),
      '#default_value' => $date_start,
      '#states' => array(
        'visible' => array(
          'input[name="auto"]' => array('checked' => FALSE),
        ),
      ),
    );

    // Show optional End Date field if this is a new subscription.
    if (empty($aid)) {
      $form['subs_end'] = array(
        '#type' => 'fieldset',
        '#title' => t('End date'),
        '#collapsible' => FALSE,
        '#weight' => 3,
      );

      if ($subscription->date_end != 0) {
        $date_end = date('Y-m-d', $subscription->date_end);
      }
      else {
        $date_end = date('Y-m-d', time() + $subscription_type->length);
      }

      $date_end = explode('-', $date_end);
      $date_end = array(
        'year' => (int) $date_end[0],
        'month' => (int) $date_end[1],
        'day' => (int) $date_end[2],
      );

      $form['subs_end']['auto_end'] = array(
        '#title' => t('End automatically after the default period for this subcription type elapses.'),
        '#description' => t("Caution: If the subscription is already active and you check this, the end date will be recalculated from today. Selecting this option will override the subscription type's default length."),
        '#type' => 'checkbox',
        '#default_value' => $subscription->date_end == 0,
      );

      $form['subs_end']['date_end'] = array(
        '#type' => 'date',
        '#title' => t('End on a specific date'),
        '#default_value' => $date_end,
        '#states' => array(
          'visible' => array(
            'input[name="auto_end"]' => array('checked' => FALSE),
          ),
        ),
      );
    }
  }

  // Only show End Date if editing, subscription started and user has permission.
  if (!empty($aid) && $administer_subs && $subscription->date_start) {
    // Only provide value if already set.
    if ($subscription->date_end > 0) {
      $date_end = date('Y-m-d', $subscription->date_end);
      $date_end = explode('-', $date_end);
      $date_end = array(
        'year' => (int) $date_end[0],
        'month' => (int) $date_end[1],
        'day' => (int) $date_end[2],
      );
    }
    else {
      $date_end = NULL;
    }

    $form['subs_end'] = array(
      '#type' => 'fieldset',
      '#title' => t('End date'),
      '#collapsible' => FALSE,
      '#weight' => 3,
    );

    $form['subs_end']['display'] = array(
      '#type' => 'checkbox',
      '#title' => t('Set end date'),
      '#default_value' => isset($date_end),
    );

    $form['subs_end']['date_end'] = array(
      '#type' => 'date',
      '#title' => t('End Date'),
      '#default_value' => $date_end,
      '#states' => array(
        'visible' => array(
          'input[name="display"]' => array('checked' => TRUE),
        ),
      ),
    );
  }

  // Only show user and status fields if user has permission.
  if ($administer_subs) {
    if (isset($subscription->uid)) {
      $user = user_load($subscription->uid);
    }

    $form['uid'] = array(
      '#type' => 'textfield',
      '#title' => t('User'),
      '#autocomplete_path' => 'user/autocomplete',
      '#default_value' => isset($user) ? $user->name : '',
      '#required' => TRUE,
      '#weight' => 0,
    );

    $form['status'] = array(
      '#type' => 'select',
      '#title' => t('Status'),
      '#options' => subs_status_options_list(),
      '#default_value' => isset($subscription->status) ? $subscription->status : $subscription_type->default_status,
      '#weight' => 1,
    );
  }

  field_attach_form('subs', $subscription, $form, $form_state);

  $submit = array();
  if (!empty($form['#submit'])) {
    $submit += $form['#submit'];
  }

  $form['actions'] = array(
    '#weight' => 100,
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save subscription'),
    '#submit' => $submit + array('subs_form_submit'),
  );

  // Display delete button if we're editing the subscription and have
  // permission.
  if (!empty($aid) && subs_access('delete', $subscription)) {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#limit_validation_errors' => array(),
      '#submit' => array('subs_form_submit_delete'),
    );
  }

  return $form;
}

/**
 * Validatation callback.
 *
 * @see subs_form()
 */
function subs_form_validate(&$form, &$form_state) {
  $subscription = $form_state['subscription'];
  $values = & $form_state['values'];

  // If we have a "uid" field submitted, the subscription is being created on
  // behalf of a user.
  if (isset($values['uid'])) {
    // Check whether we have a valid user.
    if ($user = user_load_by_name($values['uid'])) {
      // If we're creating a new subscription, check whether the current user
      // has permissions.
      if (isset($subscription->is_new)) {
        // Load current user.
        $account = user_uid_optional_load();

        if (subs_access('create', $subscription, $account)) {
          form_set_value($form['uid'], $user->uid, $form_state);
        }
        else {
          form_set_error('uid', t("You can't create subscriptions of this type!"));
        }
      }
    }
    // Bummer.
    else {
      form_set_error('uid', t('Invalid user.'));
    }
  }
  // If not, a user is creating a subscription for himself.
  else {
    $user = user_uid_optional_load();
    form_set_value($form['uid'], $user->uid, $form_state);
  }

  if (isset($values['status'])) {
    // Make sure we have a valid status.
    $options = subs_status_options_list();
    if (!array_key_exists($values['status'], $options)) {
      form_set_error('status', t('Invalid subscription status.'));
    }
  }
  // If we have no status field, the user doesn't have administer subs
  // permission so just assume the default status for this type.
  else {
    $subscription_type = subs_get_types($subscription->type);
    form_set_value($form['status'], $subscription_type->default_status, $form_state);
  }

  // Make sure we don't run into a weird situation where the user selects the
  // status to "Active" but specified a start date in the future. In this case,
  // we'll flag an error and make the user specify one or another.
  if ($subscription->status != SUBS_STATUS_ACTIVE && $values['status'] == SUBS_STATUS_ACTIVE && !$values['auto']) {
    $date_start = "{$values['date_start']['year']}-{$values['date_start']['month']}-{$values['date_start']['day']}";
    if (strtotime($date_start) !== strtotime('today')) {
      form_set_error('date_start', t("You selected 'Active' for the default status but a start date that's not today. Please set the default status to 'Pending' if you don't want the subscription to start today. It will then automatically become active on the selected date."));
    }
  }
}

/**
 * Submit callback.
 * Saves a subscription.
 *
 * @see subs_form()
 */
function subs_form_submit(&$form, &$form_state) {
  $subscription = $form_state['subscription'];
  $values =& $form_state['values'];
  $date_start = & $values['date_start'];
  $date_end = & $values['date_end'];
  $date_end_orig = $values['date_end'];
  $subscription_type = subs_get_types($subscription->type);

  // Automatic start date.
  if ($values['auto'] == 1) {
    if ($values['status'] == SUBS_STATUS_ACTIVE) {
      if ($subscription->status != SUBS_STATUS_ACTIVE) {
        $date_start = REQUEST_TIME;
        // Find subscription length.
        if ($subscription_type->length > 0) {
          $date_end = $date_start + $subscription_type->length;
        }
        else {
          $date_end = 0;
        }
      }
    }
    else {
      $date_start = 0;
    }
  }
  else {
    // Convert start date to timestamp.
    $date_start = strtotime("{$date_start['year']}-{$date_start['month']}-{$date_start['day']}");

    if (!isset($subscription->is_new) && isset($values['date_end'])) {
      if ($values['display']) {
        // Convert end date to timestamp.
        $date_end = strtotime("{$date_end['year']}-{$date_end['month']}-{$date_end['day']}");
      }
      else {
        $date_end = 0;
      }
    }
  }

  unset($values['auto']);

  // If this is a new subscription and the user decided to override the end
  // date, handle it here.
  if (isset($subscription->is_new)) {
    if (!$values['auto_end']) {
      $date_end = strtotime("{$date_end_orig['year']}-{$date_end_orig['month']}-{$date_end_orig['day']}");
    }
    else {
      // Calculate end date based on start date.
      if ($subscription_type->length > 0) {
        $date_end = $date_start + $subscription_type->length;
      }
      else {
        $date_end = 0;
      }
    }

    unset($values['auto_end']);
  }

  // Finally, if the status is "Pending" and the start date is today, we should
  // manually update the status to "Active" and ajust the end date accordingly.
  if ($subscription->status == SUBS_STATUS_PENDING) {
    if ($values['date_start'] == strtotime('today')) {
      $values['status'] = SUBS_STATUS_ACTIVE;

      // Check if the end date is the default one and, if so, adjust it
      // according to the new start date.
      $date_end_default = $subscription->date_start + $subscription_type->length;
      if ($date_end_default == $subscription->date_end) {
        $date_end = $date_start + $subscription_type->length;
      }
    }
  }

  // Build entity.
  entity_form_submit_build_entity('subs', $subscription, $form, $form_state);

  // Save entity.
  subs_save($subscription);

  // Redirect.
  $subscription_uri = entity_uri('subs', $subscription);
  $form_state['redirect'] = $subscription_uri['path'];

  // Success message.
  drupal_set_message(t('Subscription %title saved.', array('%title' => entity_label('subs', $subscription))));
}

/**
 * Submit callback.
 * Redirects to subscription delete confirmation form.
 *
 * @see subs_form()
 */
function subs_form_submit_delete($form, &$form_state) {
  $subscription = $form_state['subscription'];
  $subscription_uri = entity_uri('subs', $subscription);
  $redirect_path = $subscription_uri['path'] . '/delete';
  $form_state['redirect'] = $redirect_path;
}

/**
 * Form callback.
 * Displays subscription delete confirmation form.
 *
 * @see subs_menu()
 */
function subs_delete_form($form, &$form_state, $subscription) {
  $form_state['subscription'] = $subscription;

  // Always provide entity id in the same form key as in the entity edit form.
  $form['aid'] = array('#type' => 'value', '#value' => entity_id('subs', $subscription));
  $subscription_uri = entity_uri('subs', $subscription);
  return confirm_form($form, t('Are you sure you want to delete subscription %title?', array('%title' => entity_label('subs', $subscription))), $subscription_uri['path'], t('This action cannot be undone.'), t('Delete'), t('Cancel'));
}

/**
 * Submit handler.
 * Deletes an subscription.
 *
 * @see subs_delete_form()
 */
function subs_delete_form_submit($form, &$form_state) {
  $subscription = $form_state['subscription'];

  // Delete subscription.
  subs_delete($subscription);

  // Redirect to front page.
  $form_state['redirect'] = '<front>';

  // Success message.
  drupal_set_message(t('Subscription %title deleted.', array('%title' => entity_label('subs', $subscription))));
}